import pytest
from simevopy import Environment, Genes, Organism
from math import sqrt

def custom_lifespan_callback(organism):
    return 0

def calculate_distance(x1, y1, x2, y2):
    return sqrt((x2 - x1)**2 + (y2 - y1)**2)

@pytest.fixture
def setup_environment():
    env = Environment(1000, 1000)
    dna = chr(25) * 1 + chr(30) * 1 + chr(100) * 2
    dna2 = chr(25) * 1 + chr(80) + chr(120) * 2

    env.add_organism(Organism(Genes(dna), custom_lifespan_callback), 100, 100)
    env.add_organism(Organism(Genes(dna2), custom_lifespan_callback), 80, 80)
    return env

def test_environment_simulation(setup_environment):
    env = setup_environment
    initial_distance = calculate_distance(100, 100, 80, 80)
    
    # Simulate without error
    try:
        for _ in range(100):
            env.simulate_iteration(1)
    except Exception as e:
        pytest.fail(f"Simulation failed with an exception: {e}")
    
    all_orgs = env.get_all_organisms()

    # Check organism count
    assert len(all_orgs) == 2, "There should be exactly two organisms in the environment"
    
    # Check final distance
    first_org_pos = all_orgs[0].get_position()
    second_org_pos = all_orgs[1].get_position()
    final_distance = calculate_distance(first_org_pos[0], first_org_pos[1], second_org_pos[0], second_org_pos[1])
    assert abs(initial_distance - final_distance) < 1e-4, "The distance between organisms should not change significantly"



import numpy as np
from simevopy import Environment
from utils.common import (
    distribute_food_randomly,
    remove_all_foods,
    reproduce_organisms,
    setup_base_organism,
)

def distribute_food_like_oasis(env, food_count):
    distribute_food_randomly(env, int(food_count / 4), (1600, 1600), (1800, 1800))
    distribute_food_randomly(env, int(food_count / 4), (0, 0), (2000, 2000))
    
    distribute_food_randomly(env, int(food_count / 2), (100, 100), (500, 500))
    distribute_food_randomly(env, int(food_count / 6), (1000, 1000), (1500, 1500))


env = Environment(4000, 4000, threads=4)
setup_base_organism(env, 200)

# 初始化用於存儲每代數據的列表
organism_counts = []
average_sizes = []
average_speeds = []
average_awareness = []

for i in range(10):
    print("=========================================")
    print(f"Gen {i} th")

    distribute_food_like_oasis(env, 400)

    env.simulate_iteration(
        100,
    )

    organisms = env.get_all_organisms()
    organism_count = len(organisms)
    organism_counts.append(organism_count)

    if organism_count > 0:
        sizes = np.array([org.get_size() for org in organisms])
        speeds = np.array([org.get_speed() for org in organisms])
        awareness = np.array([org.get_awareness() for org in organisms])

        average_sizes.append(np.mean(sizes))
        average_speeds.append(np.mean(speeds))
        average_awareness.append(np.mean(awareness))
    else:
        average_sizes.append(0)
        average_speeds.append(0)
        average_awareness.append(0)

    reproduce_organisms(env)
    remove_all_foods(env)



import os
import subprocess
import sys
from setuptools import setup, Extension, find_packages
from setuptools.command.install import install
from setuptools.command.build_ext import build_ext

class CMakeBuild(build_ext):
    def run(self):
        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = [
            f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extdir}",
            f"-DPYTHON_EXECUTABLE={sys.executable}"
        ]
        build_temp = self.build_temp
        if not os.path.exists(build_temp):
            os.makedirs(build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=build_temp)
        subprocess.check_call(['cmake', '--build', '.'], cwd=build_temp)

class InstallWithCMake(install):
    def run(self):
        self.run_command('build_ext')
        install.run(self)

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        super().__init__(name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="simevopy",
    version='0.2.26',
    description="SimEvo Python bindings",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    ext_modules=[CMakeExtension('simevopy')],
    cmdclass=dict(build_ext=CMakeBuild, install=InstallWithCMake),
    url='https://github.com/YJack0000/SimEvo',
    author='YJack0000',
    author_email='yjack0000.cs12@nycu.edu.tw',
    license='MIT',
)



import math
import pytest
from simevopy import Environment, Food, Genes, Organism

def distribute_food_in_circle(env, food_count=30):
    for i in range(food_count):
        angle = i * 2 * math.pi / food_count
        x = 500 + math.cos(angle) * 300
        y = 500 + math.sin(angle) * 300
        env.add_food(Food(), x, y)

@pytest.fixture
def setup_environment():
    env = Environment(1000, 1000)
    dna = chr(40) * 2 + chr(120) * 2
    org = Organism(Genes(dna))
    env.add_organism(org, 275, 275)
    distribute_food_in_circle(env, food_count=80)
    return env

def test_food_distribution(setup_environment):
    env = setup_environment
    assert len(env.get_all_foods()) == 80, "There should be exactly 80 food particles in the environment"

def test_environment_simulation(setup_environment):
    env = setup_environment
    
    # Initial organism position check
    initial_position = env.get_all_organisms()[0].get_position()
    assert initial_position == (275, 275), "Organism should start at position (275, 275)"

    # Simulate without error
    try:
        env.simulate_iteration(300)
    except Exception as e:
        pytest.fail(f"Simulation failed with an exception: {e}")
    
    # Ensure the organism has moved
    remain_foods = env.get_all_foods()
    assert len(remain_foods) == 0, "All food particles should be consumed"
